<?php

namespace App\Service;

use App\Exception;

class IntegrationService {

    private $util;
    private $integrationFactory;

    public function __construct (\App\Util\Util $util, 
                                 \App\Factory\IntegrationFactory $integrationFactory) {
        
        $this->util = $util;
        
        $this->integrationFactory = $integrationFactory;
    }
    
    public function insertSale($sale) {
        $this->integrationFactory->insertSale($sale);
    }
    
    public function insertSaleXml($saleXml) {
        $this->integrationFactory->insertSaleXml($saleXml);
    }
    
    public function insertItemSale($item) {
        $this->integrationFactory->insertItemSale($item);
    }
    
    public function insertMovCaixa($payment) {
        $this->integrationFactory->insertMovCaixa($payment);
    }
    
    public function insertTurCaixa($opening) {
        $this->integrationFactory->insertTurCaixa($opening);
    }
    
    public function updateControlFields($salesArray) {
        foreach($salesArray as $sale){
            $this->integrationFactory->updateControlField($sale);
        }
    }
    
    public function getBranchParams($branchId) {
        return $this->integrationFactory->getBranchParams($branchId);
    }
    
    public function getStandartParameters($cdfilial) {
        return $this->integrationFactory->getStandartParameters($cdfilial);
    }
    
    public function getStandartCashier($cdfilial, $vendor) {
        return $this->integrationFactory->getStandartCashier($cdfilial, $vendor);
    }
    
    public function getProductTaxes($cdfilial, $cdproduto) {
        return $this->integrationFactory->getProductTaxes($cdfilial, $cdproduto);
    }
    
    public function insertItemTax($item) {
        $params = [
            'NRSEQITIMPOS'      => '001',
            'VRIMPOPRODIT'      => 0,
            'CDIMPOSTOEX'       => null,
            'CDCSTPRODI'        => 0,
            'CDCSTPRODPC'       => 0,
            'CDCFOPPROD'        => 0,
            'VRPERCOFINS'       => 0,
            'VRPERPIS'          => 0,
            'IDMODALBASECALC'   => 0,
            'VRBASECALCREDUZ'   => 0,
            'VRIMPOPRODREDUZ'   => 0,
            'VRBASECALCICMS'    => 0,
            'VRBCREDUZICMS'     => 0,
            'VRPRBCREDUICMS'    => 0,
            'VRIMPPIS'          => 0,
            'VRIMPCOFINS'       => 0 ,
            'VRTOTTRIBIBPT'     => 0 
        ];
        
        $params['CDFILIAL']     = $item['CDFILIAL'];
		$params['CDCAIXA']      = $item['CDCAIXA'];
		$params['NRSEQVENDA']   = $item['NRSEQVENDA'];
        $params['NRSEQUITVEND'] = $item['NRSEQUITVEND'];

        $taxes = $this->getProductTaxes($item['CDFILIAL'], $item);
        
        $taxes["VRALIQPIS"]    = $this->util->trataMascaraImposto($taxes["VRALIQPIS"]);
		$taxes["VRALIQCOFINS"] = $this->util->trataMascaraImposto($taxes["VRALIQCOFINS"]);
		$taxes["VRPEALIMPFIS"] = $this->util->trataMascaraImposto($taxes["VRPEALIMPFIS"]);

       	$doDescItem = round($item['VRDESITVEND'], 2);
		$doAcreItem = round($item['VRACRITVEND'], 2);

		$baseValue               = floatval(bcmul(str_replace(',','.',strval($item['QTPRODVEND'])), str_replace(',','.',strval(bcadd(str_replace(',','.',strval($item['VRUNITVEND'])), str_replace(',','.',strval($item['VRUNITVENDCL'])), '2'))), '2'));
		$vrBaseCalcIcmsPisCofins = floatval(bcsub(str_replace(',','.',strval(bcadd(str_replace(',','.',strval($baseValue)), str_replace(',','.',strval($doAcreItem)), '2'))), str_replace(',','.',strval($doDescItem)), '2'));
        $reductionPercentage     = isset($taxes['VRPERCREDUCAO']) ? floatval($taxes['VRPERCREDUCAO']) : 0;

        //Cálcula ICMS com redução da base de cálculo
        $vrBaseCalcReduz = 0;
        if($reductionPercentage > 0) {
			$vrBaseCalcReduz = $taxes['IDTRATIMPO'] == 'A' ?
				$vrBaseCalcIcmsPisCofins - $this->util->roundABNT($this->util->percentageToAbsolute(floatval(bcsub(str_replace(',','.',strval($baseValue)), str_replace(',','.',strval($doDescItem)), '2')), $reductionPercentage), 2) :
				$vrBaseCalcIcmsPisCofins - $this->util->roundABNT($this->util->percentageToAbsolute($vrBaseCalcIcmsPisCofins, $reductionPercentage), 2);
			$vrBaseCalcReduz = $this->util->trunc($vrBaseCalcReduz);
        }
        
        $doVrImpoProdIt = $this->util->percentageToAbsolute($vrBaseCalcIcmsPisCofins, $taxes['VRPEALIMPFIS']);
		$doVrImpPis = $this->util->percentageToAbsolute($vrBaseCalcIcmsPisCofins, $taxes['VRALIQPIS']);
        $doVrImpCofins = $this->util->percentageToAbsolute($vrBaseCalcIcmsPisCofins, $taxes['VRALIQCOFINS']);
        $params['CDIMPOSTO'] = $taxes['CDIMPOSTO'];
        $params['VRPEALPRODIT'] = $taxes['VRPEALIMPFIS'];
        $params['CDIMPOSTOEX'] = $taxes['CDINTIMPOSTO'];
    	$params['CDCSTPRODI'] = $taxes['CDCSTICMS'];
       	$params['CDCSTPRODPC'] = $taxes['CDCSTPISCOF'];
        $params['CDCFOPPROD'] = $taxes['CDCFOPPFIS'];
        $params['VRPERCOFINS'] = $taxes['VRALIQCOFINS'];
        $params['VRPERPIS'] = $taxes['VRALIQPIS'];
        $params['IDMODALBASECALC'] = $taxes['IDMODALBASECALC'];
        $params['VRBASECALCREDUZ'] = $vrBaseCalcReduz;
        $params['VRIMPOPRODREDUZ'] = $this->util->roundABNT($this->util->percentageToAbsolute($vrBaseCalcReduz, $taxes['VRPEALIMPFIS']), 2);
		$params['VRBASECALCICMS'] = $taxes['IDTRATIMPO'] == 'A' ? floatval(bcsub(str_replace(',','.',strval($baseValue)), str_replace(',','.',strval($doDescItem)), '2')) : $vrBaseCalcIcmsPisCofins;
        $params['VRBCREDUZICMS'] = $reductionPercentage == 0 ? 0 : floatval(bcsub(str_replace(',','.',strval($vrBaseCalcIcmsPisCofins)), str_replace(',','.',strval($vrBaseCalcReduz)), '2'));
        $params['VRPRBCREDUICMS'] = $reductionPercentage;
        $params['VRIMPOPRODIT'] = $this->util->roundABNT($doVrImpoProdIt, 2);
        $params['VRIMPPIS'] = $this->util->roundABNT($doVrImpPis, 2);
        $params['VRIMPCOFINS'] = $this->util->roundABNT($doVrImpCofins, 2);
        $params['VRBCPISCOFINS'] = $this->util->roundABNT($vrBaseCalcIcmsPisCofins, 2);
        
        $params = $this->util->cleanNumbers($params);
        
        $this->integrationFactory->insertItemTax($params);
    }
    
    public function getProduct($externalCode) {
        return $this->integrationFactory->getProduct($externalCode);
    }
    
    public function getOpeningPos($cdfilial, $cdcaixa) {
        return $this->integrationFactory->getOpeningPos($cdfilial, $cdcaixa);
    }
    
    public function insertSangria($sangria){
        $this->integrationFactory->insertSangria($sangria);
    }
    
    public function getIntegrationPayment($paymentExt, $revenue = false) {
        return $this->integrationFactory->getIntegrationPayment($paymentExt, $revenue);
    }
    
    public function getMovementIncrement($cdfilial) {
        return $this->integrationFactory->getMovementIncrement($cdfilial);
    }
    
    public function getSalesByIds($cdfilial, $id) {
        return $this->integrationFactory->getSalesByIds($cdfilial, $id);
    }
    
    public function updateSaleId($cdfilial, $cdcaixa, $nrseqvenda, $id){
        $this->integrationFactory->updateSaleId($cdfilial, $cdcaixa, $nrseqvenda, $id);
    }
    
    public function closePos($cdfilial, $cdcaixa,  $dtfechcaix, $cdoperador, $idatuturcaixa) {
        $this->integrationFactory->closePos($cdfilial, $cdcaixa,  $dtfechcaix, $cdoperador, $idatuturcaixa);
    }

}